﻿//## Microsoft Reciprocal License (Ms-RL)
//##
//## This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.
//##
//## 1. Definitions
//##
//## The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
//##
//## A "contribution" is the original software, or any additions or changes to the software.
//##
//## A "contributor" is any person that distributes its contribution under this license.
//## 
//## "Licensed patents" are a contributor's patent claims that read directly on its contribution.
//## 
//## 2. Grant of Rights
//## 
//## (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
//## 
//## (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.
//## 
//## 3. Conditions and Limitations
//## 
//## (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
//## 
//## (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
//## 
//## (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
//## 
//## (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
//## 
//## (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
//## 
//## (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.  

//http://mvcmembership.codeplex.com/

using System.Collections.Generic;
using System.Web.Mvc;
using System.Web.Security;
using DotNetOpenId;
using DotNetOpenId.RelyingParty;

namespace AFDS_WEB.Controllers
{
	public abstract class OpenIDController_Base : Controller
	{
		protected OpenIDController_Base()
			: this( null, null ) {}

		protected OpenIDController_Base( IFormsAuthentication formsAuth, MembershipProvider provider )
		{
			FormsAuth = formsAuth ?? new FormsAuthenticationWrapper();
			Provider = provider ?? Membership.Provider;
		}

		public IFormsAuthentication FormsAuth { get; private set; }
		public MembershipProvider Provider { get; private set; }

		[HandleError( ExceptionType = typeof(OpenIdException) )]
		[HandleError( ExceptionType = typeof(System.Net.WebException) )]
		public virtual ActionResult Login( string openid_identifier )
		{
			//### set page title & declare variables
			ViewData["Title"] = "Login via OpenID";
			var errors = new List<string>();
			var openid = new OpenIdRelyingParty();

			//### stage 1: display login form to user
			if( openid.Response == null && Request.HttpMethod != "POST" )
				return View();

			if( openid.Response == null )
			{
				//### stage 2: user submitting Identifier
				Identifier id;
				if( Identifier.TryParse( openid_identifier, out id ) )
					openid.CreateRequest( openid_identifier ).RedirectToProvider();
				else
				{
					errors.Add( ErrorMessages.InvalidIdentifierSpecified );
					ViewData["errors"] = errors;
					ViewData["openid_identifier"] = openid_identifier;
					return View();
				}
			}
			else
			{
				//### stage 3: OpenID Provider sending assertion response
				switch(openid.Response.Status)
				{
					case AuthenticationStatus.Authenticated:

						//### associate openid identity to user account and login
						var userName = AssociateOpenIdIdentityToUserName( openid.Response.ClaimedIdentifier );
						if( string.IsNullOrEmpty( userName ) )
						{
							errors.Add( ErrorMessages.AssociationFailure );
							ViewData["errors"] = errors;
							ViewData["openid_identifier"] = openid.Response.ClaimedIdentifier;
							return View();
						}
						FormsAuthentication.RedirectFromLoginPage( userName, false /* remember me */ );
						break;

					case AuthenticationStatus.Canceled:
						errors.Add( ErrorMessages.CanceledAtProvider );
						ViewData["errors"] = errors;
						ViewData["openid_identifier"] = openid.Response.ClaimedIdentifier;
						return View();
					case AuthenticationStatus.Failed:
						errors.Add( ErrorMessages.UnknownFailure + openid.Response.Exception.Message );
						ViewData["errors"] = errors;
						ViewData["openid_identifier"] = openid.Response.ClaimedIdentifier;
						return View();
				}
			}
			return new EmptyResult();
		}

		public virtual ViewResult XRDS()
		{
			return View();
		}

		[NonAction]
		protected abstract string AssociateOpenIdIdentityToUserName( string openIdIdentity );

		#region Nested type: ErrorMessages

		public struct ErrorMessages
		{
			public static string AssociationFailure = "Could not associate OpenID to a user in this system.";
			public static string CanceledAtProvider = "Login canceled by OpenID provider.";
			public static string InvalidIdentifierSpecified = "You enter your OpenID Url.";
			public static string InvalidIdentityServerSpecified = "OpenID provider not on whitelist.";
			public static string UnknownFailure = "OpenID authentication failed: ";
		}

		#endregion
	}
}